#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright (C) 2013 David Callé <davidc@framli.eu>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import Unity, UnityExtras
from gi.repository import Gio, GLib
import urllib.parse
import urllib.request
import json
import gettext

APP_NAME = 'unity-scope-reddit'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Info.Reddit'
UNIQUE_PATH = '/com/canonical/unity/scope/info/reddit'
SEARCH_URI = 'https://reddit.com/'
SEARCH_HINT = _('Search Reddit')
NO_RESULTS_HINT = _('Sorry, there are no subreddits that match your search.')
PROVIDER_CREDITS = _('Powered by Reddit')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR+'service-reddit.svg'
DEFAULT_RESULT_ICON = SVG_DIR+'service-reddit.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT


c1 = {'id'      :'results',
      'name'    :_('Posts'),
      'icon'    :SVG_DIR+'group-installed.svg',
      'renderer':Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

m1 = {'id'   :'target',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m2 = {'id'   :'author',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m3 = {'id'   :'domain',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m4 = {'id'   :'subreddit',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m5 = {'id'   :'comments',
      'type' :'i',
      'field':Unity.SchemaFieldType.OPTIONAL}
EXTRA_METADATA = [m1, m2, m3, m4, m5]

def search(search, filters):
    '''
    Any search method returning results as a list of tuples.
    Available tuple fields:
    uri (string)
    icon (string)
    title (string)
    comment (string)
    dnd_uri (string)
    mimetype (string)
    category (int)
    result_type (Unity ResultType)
    extras metadata fields (variant)
    '''
    results = []
    uri = ''
    if not search:
        uri = '%shot.json?limit=25' % (SEARCH_URI)
    elif len(search) < 3:
        return results
    else:
        search_cleaned = urllib.parse.quote(search.split('#')[0])
        uri = '%sr/%s/hot.json?limit=25' % (SEARCH_URI, search_cleaned)
    print(uri)
    data = None
    try:
        hdr = { 'User-Agent' : 'unity-scope-reddit' }
        req = urllib.request.Request(url=uri, 
                                     headers=hdr)
        response = urllib.request.urlopen(req).read()
        data = json.loads(response.decode('utf-8'))
    except Exception as error:
        print(error)
        return results
    if not data or not 'data' in data:
        return results
    for d in data ['data']['children']:
        if d['data']['over_18'] is True and not '#nsfw' in search:
            ann_icon = Unity.AnnotatedIcon.new(Gio.ThemedIcon.new(DEFAULT_RESULT_ICON))
            ann_icon.props.ribbon = "NSFW"
            icon = ann_icon.to_string()
            target = ''
        else:
            if d['data']['thumbnail'] != 'self':
                icon = d['data']['thumbnail']
            else:
                icon = None
            target = d['data']['url']
        comment = d['data']['selftext']
        results.append({'uri':'%s%s' % (SEARCH_URI, d['data']['permalink']),
                        'title':d['data']['title'],
                        'icon':icon,
                        'target':target,
                        'author':d['data']['author'],
                        'subreddit':d['data']['subreddit'],
                        'comment':comment,
                        'comments':d['data']['num_comments'],
                        'domain':d['data']['domain']})
    return results


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch (Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super (MySearch, self).__init__()
        self.set_search_context (search_context)

    def do_run (self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                result_set.add_result(**i)
        except Exception as error:
            print (error)

class Preview (Unity.ResultPreviewer):

    def do_run(self):
        if self.result.metadata['target'].get_string().split('.')[-1] in ['jpg', 'png']:
            image = Gio.FileIcon.new (Gio.file_new_for_uri(self.result.metadata['target'].get_string()))
        else:
            image = Gio.FileIcon.new (Gio.file_new_for_uri(self.result.icon_hint))
        if self.result.comment != self.result.title and self.result.comment != '':
            description = self.result.comment
        elif len(self.result.title) > 23:
            description = self.result.title
        else:
            description = ''
        preview = Unity.GenericPreview.new(self.result.title, description, image)
        preview.add_info(Unity.InfoHint.new('comments', _('Comments'), None, str(self.result.metadata['comments'].get_int32())))
        preview.add_info(Unity.InfoHint.new('subreddit', _('Submitted to'), None, "r/%s" % (self.result.metadata['subreddit'].get_string())))
        if not self.result.metadata['domain'].get_string().startswith("self."):
            preview.add_info(Unity.InfoHint.new('domain', _('Source'), None, self.result.metadata['domain'].get_string()))
        preview.props.subtitle = self.result.metadata['author'].get_string()
        icon = Gio.FileIcon.new (Gio.file_new_for_path(PROVIDER_ICON))
        open_action = Unity.PreviewAction.new("open", _("View on Reddit"), icon)
        preview.add_action(open_action)
        return preview

class Scope (Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint (self):
        return SEARCH_HINT

    def do_get_schema (self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new ()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories (self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new ()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new (c['id'], c['name'],
                                          Gio.ThemedIcon.new(c['icon']),
                                          c['renderer'])
                cs.add (cat)
        return cs

    def do_get_filters (self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new ()
#        if FILTERS:
#            
        return fs

    def do_get_group_name (self):
        return GROUP_NAME

    def do_get_unique_name (self):
        return UNIQUE_PATH

    def do_create_search_for_query (self, search_context):
        se = MySearch (search_context)
        return se

    def do_create_previewer(self, result, metadata):
        rp = Preview()
        rp.set_scope_result(result)
        rp.set_search_metadata(metadata)
        return rp

def load_scope():
    return Scope()
